#include <menu/draw.h>
#include "bnr2yuv.h"
extern u32* frameBuffer;

/* Courtesy of Maximillionop */

static void RGB5A1TORGB888(u16 RGB5A1, RGBAPoint *RGBA, RGBPoint bgColor)
{
	#ifdef PCTEST
		u16 WorkPoint = ((RGB5A1 & 0xFFFF) >> 8) + ((RGB5A1 & 0xFF) << 8);
	#else
		u16 WorkPoint = RGB5A1;
	#endif
	double Alpha, bgAlpha;

	RGBA->Alpha = (((WorkPoint & (0x8000)) >> 15) == 1); // Bit 16
	if(RGBA->Alpha)
	{
		RGBA->RGB.B = (WorkPoint & 0x1F); // Bits 1 to 5
		RGBA->RGB.B = ((RGBA->RGB.B << 3) | (RGBA->RGB.B >> 2));

		RGBA->RGB.G = (WorkPoint & 0x03E0) >> 5; // Bits 6 to 10
		RGBA->RGB.G = ((RGBA->RGB.G << 3) | (RGBA->RGB.G >> 2));

		RGBA->RGB.R = (WorkPoint & 0x7C00) >> 10; // Bits 11 to 15
		RGBA->RGB.R = ((RGBA->RGB.R << 3) | (RGBA->RGB.R >> 2));
	} else {
		Alpha = (WorkPoint & 0x7000) >> 12; // Bits 13 to 15
		Alpha = (Alpha / 7.0);
		bgAlpha = 1.0 - Alpha;

		RGBA->RGB.B = (WorkPoint & 0x0F); // Bits 1 to 4
		RGBA->RGB.B = (u8)((double)(16 * RGBA->RGB.B) * Alpha + (double)(bgColor.B) * bgAlpha);

		RGBA->RGB.G = (WorkPoint & 0xF0) >> 4; // Bits 5 to 8
		RGBA->RGB.G = (u8)((double)(16 * RGBA->RGB.G) * Alpha + (double)(bgColor.G) * bgAlpha);

		RGBA->RGB.R = (WorkPoint & 0x0F00) >> 8; // Bits 9 to 12
		RGBA->RGB.R = (u8)((double)(16 * RGBA->RGB.R) * Alpha + (double)(bgColor.R) * bgAlpha);
	}
}

static void RGB888TOYCBYCr(RGBAPoint RGBA1, RGBAPoint RGBA2, YCbYCrPoint *YCbYCr)
{
	long tR,tG,tB;
	tR = (RGBA1.RGB.R+RGBA2.RGB.R)/2;
	tG = (RGBA1.RGB.G+RGBA2.RGB.G)/2;
	tB = (RGBA1.RGB.B+RGBA2.RGB.B)/2;
	YCbYCr->Y1 = (u8)((0.29900 * RGBA1.RGB.R) + (0.58700 * RGBA1.RGB.G) + (0.11400 * RGBA1.RGB.B));
	YCbYCr->Y2 = (u8)((0.29900 * RGBA2.RGB.R) + (0.58700 * RGBA2.RGB.G) + (0.11400 * RGBA2.RGB.B));
	YCbYCr->Cb = (u8)((-0.16874 * tR) - (0.33126 * tG) + (0.50000 * tB) + 0x80);
	YCbYCr->Cr = (u8)((0.50000 * tR) - (0.41869 * tB) - (0.08131 * tB) + 0x80);
}

void ConvertBanner(BannerData RGB5A1Array, RGBPoint BgColor,int x, int y, int zoom)
{
	RGBAPoint RGBAArray[32][96];
	YCbYCrPoint YCbYCrArray[32][48];
	int X, Y; // Image coordinates
	int TileX, TileY; // In-Tile coordinates
	memset(YCbYCrArray,0,sizeof(YCbYCrArray));
	memset(RGBAArray,0,sizeof(RGBAArray));
	for(Y = 0; Y < 32; Y++)
	{
		for(X = 0; X < 96; X++)
		{
			TileX = X % 4;
			TileY = Y % 4;
			RGB5A1TORGB888(RGB5A1Array[(Y - TileY) / 4][(X - TileX) / 4][TileY][TileX], &RGBAArray[Y][X], BgColor);
			if(X % 2) //every odd pixel
			{
				RGB888TOYCBYCr(RGBAArray[Y][X-1], RGBAArray[Y][X], &YCbYCrArray[Y][(X - 1) / 2]);		
			}
		}
	}
	
	// Draw YCbYCrArray, its a 48*32 array containing the whole image
	//menuDrawBanner(frameBuffer,YCbYCrArray,75,120,2);
	menuDrawBanner(frameBuffer,YCbYCrArray,x,y,zoom);

}
