#include <linux/string.h>
#include <types.h>
#include <console.h>
#include <context.h>
#include <irq.h>
#include <exception.h>
#include <cache.h>


static const u32 _dvdlowreset_org[12] = {
	0x7C0802A6,0x3C80CC00,0x90010004,0x38000002,0x9421FFE0,0xBF410008,0x3BE43000,0x90046004,
	0x83C43024,0x57C007B8,0x60000001,0x941F0024
};

static const u32 _dvdlowreset_new[8] = {
	0x3FE08000,0x63FF1800,0x7FE803A6,0x4E800021,0x4800006C,0x60000000,0x60000000,0x60000000
};

void dvd_patchreset(void *addr,u32 len)
{
	void *copy_to;
	void *addr_start = addr;
	void *addr_end = addr+len;

	if(len<sizeof(_dvdlowreset_org)) return;

	while(addr_start<addr_end) {
		if(memcmp(addr_start,_dvdlowreset_org,sizeof(_dvdlowreset_org))==0) {
			// we found the DVDLowReset
			copy_to = (addr_start+0x18);
			printf("\npatching DVD Reset");
			memcpy(copy_to,_dvdlowreset_new,sizeof(_dvdlowreset_new));
		}
		addr_start += 4;
	}
}

static const u32 _dvdlowreadid_org[10] = {
	0x7C0802A6,0x39000000,0x90010004,0x3CA0A800,0x38050040,0x9421FFE8,0x38C00020,0x3CA08000,
	0x93E10014,0x93C10010	
};

static const u32 _dvdlowreadid_new[4] = {
	0x48000098,0x60000000,0x60000000,0x60000000
};

void dvd_patchreadid(void *addr,u32 len)
{
	void *copy_to;
	void *addr_start = addr;
	void *addr_end = addr+len;

	if(len<sizeof(_dvdlowreadid_org)) return;

	while(addr_start<addr_end) {
		if(memcmp(addr_start,_dvdlowreadid_org,sizeof(_dvdlowreadid_org))==0) {
			// we found the DVDLowReadid
			copy_to = (addr_start+0x4);
			printf("\npatching LOW DVD Read ID");
			memcpy(copy_to,_dvdlowreadid_new,sizeof(_dvdlowreadid_new));
		}
		addr_start += 4;
	}
}
