#include "asm.h"
/*
	Simple Game reloader which runs at runtime in a retail game

	v1.0 final.
	- added re-patch code at runtime
	- added region setup
	- added streaming audio
	
	initial beta.
	- working mostly
	
	emu_kidid '07
*/
	.section .text

	.globl _start
_start:

	# Perform drive reset
	lis		r4 , 0xcc00							# setup reset register (high)
	ori		r4 , r4 , 0x3024						# setup reset register (low)
	lwz		r5 , 0(r4)							# read reset register
	rlwinm	r5 , r5 , 0 , 30 , 28					# clear bit 2
	ori		r5 , r5 , 1							# set bit 0
	stw		r5 , 0(r4)							# write back to reset register
	lwz		r5 , 0(r4)							# read reset register
	ori		r5 , r5 , 4							# set bit 2
	ori		r5 , r5 , 1							# set bit 0
	stw		r5 , 0(r4)							# write back to reset register -> reset DVD drive

	# Wait some time (~ 8 secs)
	li		r3 , 8000								# ~ 8 seconds
	bl		Delay								# ... delay
	
	bl		_readdiscid					# read the disc id
	bl		_setupregion
	bl		LoadApploader			

	blr									# never reach here

	.globl Delay
Delay:
	lis		r4 , 0x0003							# 0x30000 for the inner delay loop

DelayLoopInner:
	subi		r4 , r4 , 1							# decrement within inner loop
	cmpwi	r4 , 0								# zero reached ?
	bne		DelayLoopInner							# => no, back to inner loop
	subi		r3 , r3 , 1							# decrement given delay time
	cmpwi	r3 , 0								# zero reached ?
	bne		Delay								# => no, back to outer loop
	blr											# delay performed, back to caller
	
	.globl _setupregion
_setupregion:

	lis		r4,0x8000
	ori		r4,r4,0x0003
	lbz		r6,0(r4)		#load 0x03 from DVD (now in memory) to r6
	
	cmpwi	r6 , 0x45		# == 'E' for NTSC?
	beq		setNTSC			# => Yes, set NTSC and return.

	cmpwi	r6 , 0x50		# == 'P' for PAL?
	beq		setPAL			# => Yes, set PAL and return.
	
	cmpwi	r6 , 0x4A		# == 'J' for JAP?
	beq		setNTSC			# => Yes, set JAP and return.
	
	cmpwi	r6 , 0x44		# == 'D' for Other(PAL60)?
	beq		setPAL60		# => Yes, set PAL60 and return.
	
	setNTSC:
	lis		r5,0x8000
	ori		r5,r5,0x00CC	#0x800000CC (Video Value)
	li		r19,0
	stb		r19,0(r5)		#0x800000CC == 0;
	blr
	
	setPAL:
	lis		r5,0x8000
	ori		r5,r5,0x00CC	#0x800000CC (Video Value)
	li		r19,1
	stb		r19,0(r5)		#0x800000CC == 1;
	blr
	
	setPAL60:
	lis		r5,0x8000
	ori		r5,r5,0x00CC	#0x800000CC (Video Value)
	li		r19,5
	stb		r19,0(r5)		#0x800000CC == 5;
	blr
	
	blr
	
	
LoadApploader:
	lis		r9 , 0x8010							# read 32 bytes (apploader header) from DVD offset 0x2440 to GC memory 0x80100000
	mr		r3 , r9								# -"-
	li		r4 , 0x2440							# -"-
	li		r5 , 32								# -"-
	bl		DVDReadSector							# -"-

	lis		r3 , 0x8120							# read apploader from DVD offset 0x2460 to 0x81200000 with length given in header at position 20
	li		r4 , 0x2460							# -"-
	lwz		r5 , 20(r9)							# -"-
	addi		r5 , r5 , 32							# -"-
	bl		DVDReadSector							# -"-

	lis		r12 , ApploaderData1@h					# jump to entryfunction of apploader given in header at postion 16
	ori		r12 , r12 , ApploaderData1@l				# (and receive three functionpointers for 'init' , 'main' and 'final' calling of apploader)
	mr		r3 , r12								# -"-
	addi		r12 , r12 , 4							# -"-
	mr		r4 , r12								# -"-
	addi		r12 , r12 , 4							# -"-
	mr		r5 , r12								# -"-
	lwz		r8 , 16(r9)							# -"-
	mtctr	r8									# -"-
	bctrl										# -"-

LoadApploader_Loop1:
	lis		r12 , ApploaderData1@h					# read out DVD offset , length and GC memory position for each chunk to load
	ori		r12 , r12 , ApploaderData1@l				# (using the 'main' function call in the apploader)
	lis		r15 , ApploaderData2@h					# -"-
	ori		r15 , r15 , ApploaderData2@l				# -"-
	mr		r3 , r15								# -"-
	addi		r15 , r15 , 4							# -"-
	mr		r4 , r15								# -"-
	addi		r15 , r15 , 4							# -"-
	mr		r5 , r15								# -"-
	lwz		r8 , 4(r12)							# -"-
	mtctr	r8									# -"-
	bctrl										# -"-

	cmpwi	r3 , 0								# 'main' function returned zero (= no more chunks to load) ?
	beq		LoadApploader_End						# => yes, finish chunk loading

	lis		r15 , ApploaderData2@h					# use DVD offset , length and GC memory position to read a chunk from the DVD
	ori		r15 , r15 , ApploaderData2@l				# -"-
	lwz		r3 , 0(r15)							# -"-
	lwz		r4 , 8(r15)							# -"-
	lwz		r5 , 4(r15)							# -"-
	bl		DVDReadSector							# -"-

	b		LoadApploader_Loop1						# loop for next chunk to read

LoadApploader_End:
	lis		r12 , ApploaderData1@h					# use 'final' function call in the apploader to receive final application entrypoint
	ori		r12 , r12 , ApploaderData1@l				# -"-
	lwz		r8 , 8(r12)							# -"-
	mtctr	r8									# -"-
	bctrl
	
	mtlr		r3									# write final application entrypoint into link register
	blr											# and jump ... !

DVDReadSector:
	lis		r13 , 0xcc00							# setup DVD base register (high)
	ori		r13 , r13 , 0x6000						# setup DVD base register (low)
	lwz		r14 , 0(r13)							# read DVD status 1
	ori		r14 , r14 , 0x14						# set bit 2 and 4
	stw		r14 , 0(r13)							# write back to DVD status 1
	li		r14 , 0								# zero to ...
	stw		r14 , 4(r13)							# ... DVD status 2
	lis		r14 , 0xa800							# setup ReadSector command (A8000000)
	stw		r14 , 8(r13)							# write to DVD command 1
	srawi	r4 , r4 , 2							# correct DVD offset value ( >> 2 )
	stw		r4 , 12(r13)							# write to DVD command 2
	stw		r5 , 16(r13)							# write datalength of chunk to DVD command 3 ...
	stw		r5 , 24(r13)							# ... and to DVD DMA length register
	stw		r3 , 20(r13)							# write GC memoryaddress of chunk to DVD DMA address register
	li		r14 , 3								# DMA mode (3)
	stw		r14 , 28(r13)							# startup command

DVDReadSector_Loop1:
	lwz		r14 , 24(r13)							# read DVD DMA length register
	cmpwi	r14 , 0								# is zero ?
	bne		DVDReadSector_Loop1						# => no, check again

	mflr		r11									# backup link register
	mr		r4 , r5								# Invalidate cache of the memory chunk
	bl		InvalidateCache						# -"-
	mtlr		r11									# restore link register
	blr											# return to caller

# ------------------------------------------------------------------------------------------------------------------------------------

InvalidateCache:
	srawi	r4 , r4 , 5							# correct given size of range
	mtctr	r4									# write to counter register

InvalidateCache_Loop1:	
	dcbf		r0 , r3								# invalidate instruction cache (32 byte block)
	icbi		r0 , r3								# invalidate data cache (32 byte block)
	addi		r3 , r3 , 32							# increment range by 32
	bdnz		InvalidateCache_Loop1					# counter reached zero ? => no, do another round
	blr											# return to caller

# ------------------------------------------------------------------------------------------------------------------------------------

.align 4
ApploaderData1:
	.long	0 , 0 , 0								# 'init' , 'main' , 'final' apploader function call pointer
ApploaderData2:
	.long	0 , 0 , 0								# 'memory destination' , 'length' , 'DVD offset' of each chunk to load from the apploader

		
	###################### ANACONDA04 CODE FROM UTOPIA ############################
    .globl _readdiscid
_readdiscid:
    lis		r3 , 0xcc00							# setup DVD base register (high)
	ori		r3 , r3 , 0x6000					# setup DVD base register (low)
	lwz		r4 , 0(r3)							# read DVD status 1
	ori		r4 , r4 , 0x14						# set bit 2 and 4
	stw		r4 , 0(r3)							# write back to DVD status 1
	li		r4 , 0								# zero to ...
	stw		r4 , 4(r3)							# ... DVD status 2
	lis		r4 , 0xa800							# setup ReadDiscID command (A8000040)
	ori		r4 , r4 , 0x0040					# -"-
	stw		r4 , 8(r3)							# write to DVD command 1
	lis		r4 , 0								# zero to ...
	stw		r4 , 12(r3)							# ... DVD command 2
	li		r4 , 0x20							# 32 bytes (Disc ID length) to ...
	stw		r4 , 16(r3)							# ... DVD command 3
	stw		r4 , 24(r3)							# ... and to DVD DMA length register
	lis		r4 , 0x8000							# Gamecube memory address 0x80000000 for the buffer
	stw		r4 , 20(r3)							# write to DVD DMA address register
	li		r4 , 3								# DMA mode (3)
	stw		r4 , 28(r3)							# startup command

1:	lwz		r4 , 0(r3)							# read DVD status 1 register
	andi.	r4 , r4 , 0x14						# mask bit 2 and 4 (error and finish)
	cmpwi	r4 , 0								# none set ?
	beq+	1b          						# => yeah, so check again

	mflr		r10								# backup link register
	lis		r3 , 0x8000							# invalidate cache (32 bytes) at 0x80000000
	li		r4 , 32								# -"-
	bl		InvalidateCache						# -"-
	mtlr		r10								# restore link register

	blr											# return to caller
    
   
_end:
