/*
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <console.h>
#include <time.h>
#include <cache.h>
#include <dvd.h>
#include <linux/string.h>
#include <font.h>
#include <vsprintf.h>
#include <GC_Dvd.h>
#include <anaconda.h>
#include <menu/menu.h>
#include <menu/draw.h>
#include <bnr2yuv.h>
#include <pad.h>
#include <menu/banner.h>
#include <fst.h>

volatile unsigned long* dvd = (volatile long*)0xCC006000;
extern u32* frameBuffer;

unsigned int dvd_readqoob(void* dst, int len, unsigned int offset)
{
	if ((((int)dst) & 0xC0000000) == 0x80000000) // cached?
		dvd[0] = 0x2E;
	dvd[1] = 0;

	dvd[2] = 0xA8000000;
	dvd[3] = offset >> 2;
	dvd[4] = len;
	dvd[5] = (unsigned long)dst;
	dvd[6] = len;
	dvd[7] = 3; // enable reading!
	dcache_inv(dst, len);
	while (dvd[7] & 1);

	if (dvd[0] & 0x4)
		return 1;
	return 0;
}
int read_sector(void* buffer, int sector)
{
	return dvd_readqoob(buffer, 2048, sector * 2048);
}

int read_safe(void* dst, int offset, int len)
{
	int ol = len;
	while (len)
	{
		int sector = offset / 2048;
		read_sector(sector_buffer, sector);
		// write_font(450,500,"Reading Disc..");	
		int off = offset & 2047;

		int rl = 2048 - off;
		if (rl > len)
			rl = len;
		memcpy(dst, sector_buffer + off, rl);	

		offset += rl;
		len -= rl;
		dst += rl;
	}
	return ol;
}

void dvd_reset(void)
{
	int i;
        *(unsigned long*)0xcc006004 = 2;
        unsigned long v = *(unsigned long*)0xcc003024;
        *(unsigned long*)0xcc003024 = (v &~4) | 1;

	for ( i = 0; i < 10000; i++ );
        *(unsigned long*)0xcc003024 = v | 5;
}

unsigned int dvd_read(void* dst, int len, unsigned int offset)
{
	unsigned char sector[2048 + 0x40];
	unsigned char* buffer = (unsigned char*)((unsigned int)sector & 0xffffffe0);

	if (len > 2048)
		return 1;

		dvd[0] = 0x2E;
		dvd[1] = 0;	
		dvd[2] = 0xA8000000;
		dvd[3] = offset >> 2;
		dvd[4] = len + 0x1f;
		dvd[5] = (unsigned long)buffer;
		dvd[6] = len + 0x1f;
		dvd[7] = 3; // enable reading!	
		while (dvd[7] & 1);
		dcache_inv(buffer, len);
		memcpy(dst, buffer + (offset & 0x3), len);
		dcache_flush(buffer,len);

	if (dvd[0] & 0x4)
		return 1;
	return 0;
}


unsigned int dvd_read64(void* dst, int len, unsigned int offset)
{
	unsigned char sector[2048 + 0x40];
	unsigned char* buffer = (unsigned char*)((unsigned int)sector & 0xffffffe0);

	if (len > 2048)
		return 1;

		dvd[0] = 0x2E;
		dvd[1] = 0;	
		dvd[2] = 0xA8000000;
		dvd[3] = offset;
		dvd[4] = len + 0x1f;
		dvd[5] = (unsigned long)buffer;
		dvd[6] = len + 0x1f;
		dvd[7] = 3; // enable reading!	
		while (dvd[7] & 1);
		dcache_inv(buffer, len);
		memcpy(dst, buffer + (offset & 0x3), len);
		dcache_flush(buffer,len);

	if (dvd[0] & 0x4)
		return 1;
	return 0;
}

unsigned int dvd_read_id()
{
	dvd[0] = 0x2E;
	dvd[1] = 0;
	dvd[2] = 0xA8000040;
	dvd[3] = 0;
	dvd[4] = 0x20;
	dvd[5] = 0x80000000;
	dvd[6] = 0x20;
	dvd[7] = 3; // enable reading!

	while (dvd[7] & 1);

	if (dvd[0] & 0x4)
		return 1;
	return 0;
}

void dvd_motor_off()
{
	dvd[0] = 0x2E;
	dvd[1] = 0;

	dvd[2] = 0xe3000000;
	dvd[3] = 0;
	dvd[4] = 0;
	dvd[5] = 0;
	dvd[6] = 0;
	dvd[7] = 1; // enable reading!
	while (dvd[7] & 1);
}

unsigned int dvd_init()
{
	if (dvd_read_id())
		return -1;
	dcache_inv((void*)0x80000000, 0x20);
	return 0;
}

unsigned int dvd_get_error(void)
{
	dvd[2] = 0xE0000000;
	dvd[8] = 0;
	dvd[7] = 1;
	while (dvd[7] & 1);
	return dvd[8];
}

void dvd_set_streaming(void)
{
	long dvdheader[0x20];
	memcpy(dvdheader,(void*)0x80000000,0x20);
	if(dvdheader[2] == 0x01000000)
	{
		dvd[0] = 0x2E;
		dvd[2] = 0xE4010000;
		dvd[7] = 1;
		while (dvd[7] & 1);
	}
	else
	{		
		dvd[0] = 0x2E;
		dvd[2] = 0xE4000000;
		dvd[7] = 1;
		while (dvd[7] & 1);
	}
}

unsigned int dvd_get_diconfig(void)
{
	return dvd[9];
}

const char* dvd_get_error_message(void)
{
	switch (dvd_get_error() >> 24)
	{
		/* as documented in NinDoc */
	case 0x00:
		return "Ready.";
	case 0x01:
		return "Cover is opened.";
	case 0x02:
		return "Disk change.";
	case 0x03:
		return "No Disk.";
	case 0x04:
		return "Motor stop.";
	case 0x05:
		return "Disk ID not read.";
	default:
		return "Modified";
	}
}

/* FST */



long BNROffset = 0;
	
unsigned int dvd_read_2(void* dst, unsigned int len, unsigned int offset)
{
	int notdone = 1;
	unsigned int tempoffset = 0;
	unsigned int templength = 0;
	unsigned int addrfix = 0;
	templength = len;
	tempoffset = offset;
	if(templength<=2048) {
		dvd_read(dst,len,offset);
		return 0;
	}	
	
	while(notdone) {
		if(templength<=2048) {
			dvd_read(dst+addrfix,templength,tempoffset);
			return 0;
		}
		else {
			dvd_read(dst+addrfix,2048,tempoffset);
			tempoffset+=2048;
			templength-=2048;
			addrfix+=2048;
		}
	}	
	
}

// OLD Functions 

/* FST */

u32 fst_size()
{
	long fstsize = 0;
	read_safe(&fstsize, 0x428, 4); //Size of the FST
	return fstsize;
}

u32 fst_location()
{
	long fstptr = 0;
	read_safe(&fstptr,0x424, 4); //offset of the FST ("fst.bin")
	return fstptr;
}

//extern long BNROffset = 0;

void getBannerOffset()
{
	long FST[700*1024];
	long* buffer __attribute__((aligned(32))) = (long*)FST;
	long fileSize = 0,fileOffset = 0;
	long fileptr = 0, currentEntry = 0;
	long tempRead[2048];
	int u = 0,d=0;
	long singleEntry[3];
	u32 z,l,strTblOffset;
	z = fst_size();
	l = fst_location();
	d = l;
	/* Read the FST */
	while(d < l+z)
	{
		dvd_read(buffer, 2048, d);
		buffer+=2048;
		d+=2048;
	}
	/* Needs to be set for AR? */
	*(long*)0x80000038 = &FST;
	
	strTblOffset = FST[2]*12;//from the start of the FST
	d = 0;
	while(fileptr < strTblOffset)
	{
		fileptr = d*3;
		
		for(u = 0; u<3; u++) //copy 12bytes (3x4bytes)
			singleEntry[u] = FST[fileptr+u];
		if(singleEntry[0] <= 0x00ffffff) //file
		{
			if(singleEntry[2] == 0x00001960)
			{
				dvd_read(tempRead,2048,singleEntry[1]);
				if(tempRead[0] == 0x424E5231)
					BNROffset = singleEntry[1];
			}
			if(singleEntry[2] == 0x00001fa0)
			{
				dvd_read(tempRead,2048,singleEntry[1]);
				if(tempRead[0] == 0x424E5232)
					BNROffset = singleEntry[1];
			}
		}
		d++;
	}

}

void showBanner(int x, int y, int zoom)
{	
	BannerData Banner;
	RGBPoint BackgroundColor;
	char banner[8192];
	BNROffset = 0;
	// read in the file then copy the needed gfx data to our array 
	getBannerOffset();
	if(BNROffset == 0)
		memcpy(banner,blankbanner,0x1960);
	else
	{
		read_safe(&banner,BNROffset,0x1960);
		memcpy(&TEMPBanner,&banner,0x1960);
	}
	memcpy(Banner, banner+0x20, 6144);
	
	BackgroundColor.R = 0;
	BackgroundColor.G = 0;
	BackgroundColor.B = 0;
	ConvertBanner(Banner, BackgroundColor,x,y,zoom);
}
