/*
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include "time.h"
#include "lwip/debug.h"

#include "lwip/mem.h"
#include "lwip/memp.h"
#include "lwip/sys.h"

#include "lwip/stats.h"

#include "lwip/ip.h"
#include "lwip/udp.h"
#include "lwip/tcp.h"
#include "lwip/arch/gc/include/netif/gcif.h"
#include "netif/etharp.h"

#include "dolup.h"
#include "irq.h"

#include "console.h"

extern void httpd_start();
struct ip_addr* sram_get_ip(int w);

static struct netif netif;
static tb_t now, start;

void network_init()
{
	struct ip_addr* ipaddr = sram_get_ip(0),* netmask = sram_get_ip(1),* gw = sram_get_ip(2);

#ifdef STATS
	stats_init();
#endif /* STATS */

	mem_init();
	memp_init();
	pbuf_init(); 
	netif_init();
	ip_init();
	//	udp_init();
	tcp_init();
	etharp_init();

	if (!netif_add(&netif, ipaddr, netmask, gw, NULL, gcif_init, ip_input))
	{
		//printf("netif_add failed!\n");
		return;
	}

	netif_set_default(&netif);

	dolup_init();
	httpd_start();
//	ftpd_start();
	mftb(&start);
}

void network_reinit()
{
	struct ip_addr* ipaddr = sram_get_ip(0),* netmask = sram_get_ip(1),* gw = sram_get_ip(2);

	//Set new network addresses
	netif_set_addr(&netif, ipaddr, netmask, gw);
}

void network_poll()
{
	mftb(&now);
	if (tb_diff_msec(&now, &start) >= 250)
	{
		unsigned long flags;
		mftb(&start);
		cli(flags);
		tcp_tmr();
		sti(flags);
	}
}
