/*
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include "ipl.h"
#include "pad.h"
#include "lwip/ip_addr.h"
#include "lwip/inet.h"
#include "processor.h"
#include "exi.h"
#include "console.h"
#include "pad.h"
#include "ipl.h"
#include "video.h"
#include "color.h"
#include "time.h"
#include "sram.h"
#include <linux/string.h>
#include <context.h>
#include <irq.h>
#include <exception.h>
#include <cache.h>
#include "font.h"
#include "main.h"
extern u32* frameBuffer;
void sram_set_videomode(int v)
{
	sram_data[33] = v;
}

int sram_get_videomode(void)
{
	return sram_data[33];
}

void sram_set_autoboot(int v)
{
	sram_data[0x3d] = v;
}

int sram_get_autoboot(void)
{
	return sram_data[0x3d];
}
void sram_set_patchcode(int v)
{
	sram_data[0x3c] = v;
}

int sram_get_patchcode(void)
{
	return sram_data[0x3c];
}

struct ip_addr* sram_get_ip(int w)
{
	return (struct ip_addr *)(sram_data + 0x24 + w * 4);
}

const char* patchSetting()
{
	int z = sram_get_patchcode();
	if (z == 0)
		return "Disc Swap Enabled"; 
	if (z == 1)
		return "Disc Swap Disabled"; 
	if (z == 2)
		return "Restart Code(BETA)"; 
	if (z == 3)
		return "Disabled"; 
}

const char* autoDetect()
{
	int z = sram_get_autoboot();
	if (z == 0)
		return "Always Ask"; 
	if (z == 1)
		return "Force Autodetect"; 
	if (z == 2)
		return "Force NTSC"; 
	if (z == 3)
		return "Force PAL"; 
	if (z == 4)
		return "Force PAL60";
	if (z == 5)
		return "Force 480p"; 
}

const char* curLang()
{		
	int z = sram_get_value(0x12);
	if (z == 0)
		return "English"; 
	if (z == 1)
		return "German"; 
	if (z == 2)
		return "French"; 
	if (z == 3)
		return "Spanish"; 
	if (z == 4)
		return "Italian";
	if (z == 5)
		return "Dutch";
}

void sram_load_defaults(void)
{
	sram_data[32] = '!';

	sram_set_autoboot(0);
	sram_set_videomode(1);
	sram_set_patchcode(0);
	IP4_ADDR(sram_get_ip(0), 10, 1, 1, 64);
	IP4_ADDR(sram_get_ip(1), 255, 0, 0, 0);
	IP4_ADDR(sram_get_ip(2), 10, 1, 1, 1);
}

void read_sram(void)
{
	//TODO: this somewhere else
	struct PAD pad;
	PAD_ReadState(&pad, 0);

	if (!pad.Digital.Start)
		exi_sram_read(sram_data);
	else
		memset(sram_data + 32, 0, 32);

	if (sram_data[32] != '!')
		sram_load_defaults();
}

void write_sram(void)
{
	sram_update_crcs();
	
}

int sram_get_value(int v)
{
	return sram_data[v];
}

void sram_set_value(int w, int v)
{
	sram_data[w] = v;
	write_sram();
}

void checksums(unsigned short *buf, unsigned short *c1, unsigned short *c2) 
{ 
	int i; 
    *c1 = 0; *c2 = 0; 
    for (i = 0;i<4;++i) 
    { 
        *c1 += buf[0x06 + i]; 
        *c2 += (buf[0x06 + i] ^ 0xFFFF); 
    } 

} 

u32 sram_rtc_bias()
{
	u32 sramBuf[16];
	memcpy(&sramBuf,sram_data,16);
	return sramBuf[3];
}

void sram_set_rtc_bias(long ticks)
{
	u32 sramBuf[16];
	memcpy(&sramBuf,sram_data,16);
	sramBuf[3]+=ticks;
	memcpy(&sram_data,sramBuf,64);
}

void sram_update_crcs()
{
	unsigned short crc1 = 0, crc2 = 0, sramBuf[64];
	memcpy(&sramBuf,sram_data,64);
	checksums(sramBuf, &crc1, &crc2);
	sramBuf[0] = crc1;
	sramBuf[1] = crc2;
	exi_sram_write(sramBuf);
}

void sram_set_progressive()
{
	sram_data[0x13] = 0xbc;
	write_sram();
}