/*
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */
#include "video.h"
#include "font.h"
#include "main.h"
#include <pad.h>
#define MEM_VIDEO_BASE  			 (0xCC002000)
#define MEM_VIDEO_BASE_PTR  		 (u32*)MEM_VIDEO_BASE

#define R_VIDEO_DCR 				 *(vu16*)(MEM_VIDEO_BASE+0x02)
#define R_VIDEO_FRAMEBUFFER_1   	 *(vu32*)(MEM_VIDEO_BASE+0x1C)
#define R_VIDEO_FRAMEBUFFER_2   	 *(vu32*)(MEM_VIDEO_BASE+0x24)
#define R_VIDEO_HALFLINE_1  		 *(vu16*)(MEM_VIDEO_BASE+0x2C)
#define R_VIDEO_HALFLINE_2  		 *(vu16*)(MEM_VIDEO_BASE+0x2E)
#define R_VIDEO_STATUS  			 *(vu16*)(MEM_VIDEO_BASE+0x6C)

static const unsigned long video_ntsc[32] =
{
	0x0F060001, 0x476901AD, 0x02EA5140, 0x00030018, 0x00020019, 0x410C410C, 0x40ED40ED, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x110701AE, 0x10010001, 0x00010001, 0x00010001, 0x00000000, 0x00000000, 0x28500100, 0x1AE771F0, 0x0DB4A574, 0x00C1188E, 0xC4C0CBE2, 0xFCECDECF, 0x13130F08, 0x00080C0F, 0x00FF0000, 0x00000000, 0x02800000, 0x000000FF, 0x00FF00FF, 0x00FF00FF
};

static const unsigned long video_pal[32] =
{
	0x11F50101, 0x4B6A01B0, 0x02F85640, 0x00010023, 0x00000024, 0x4D2B4D6D, 0x4D8A4D4C, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x013C0144, 0x113901B1, 0x10010001, 0x00010001, 0x00010001, 0x00000000, 0x00000000, 0x28500100, 0x1AE771F0, 0x0DB4A574, 0x00C1188E, 0xC4C0CBE2, 0xFCECDECF, 0x13130F08, 0x00080C0F, 0x00FF0000, 0x00000000, 0x02800000, 0x000000FF, 0x00FF00FF, 0x00FF00FF
};

void vi_init(int mode, unsigned long fbaddr)
{
	int i = 0;
	unsigned long *ptr = 0;
	*(volatile unsigned long*)0xCC002000 = 0x00000003;
	
	if (mode == 0)
	{
		ptr = video_ntsc;
	}
	if (mode == 1)
	{
		ptr = video_pal;
	}
	
	ptr[7] = fbaddr;
	ptr[9] = fbaddr + 1280;
	
	for (i = 0; i < 32; ++i)
	{
		((unsigned long*)0xCC002000)[i] = ptr[i];
	}
}

void video_wait_vsync(void)
{
	while (R_VIDEO_HALFLINE_1 >= 200);
	while (R_VIDEO_HALFLINE_1 < 200);
}

int video_get_mode(void)
{
	return (R_VIDEO_DCR & 0x30) >> 8;
}

void Video_Init()
{

	char IsPal = 0;
	int i = 0;
	static char* videosys[] ={"NTSC", "PAL"};
	u16 oldVI[0x80];
	memcpy(oldVI,(void*)0xCC002000,0x80);
	if(oldVI[1] == 0x0101){ 
		IsPal = 1;
	}
	else
		IsPal = 0;	
	if (IsPal) {
		*(unsigned long*)0x800000cc = 1;
		vi_init(1, MEM_FB);
	}
	else {
		*(unsigned long*)0x800000cc = 0;
		vi_init(0, MEM_FB);
	}
	sprintf(CubeInfoStr1, "%s BIOS Revision 1.0", videosys[IsPal]);

}