/*
 * different stuff to access the IPL chips (including special functions)
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */
#include <exi.h>
#include <time.h>
#include <ipl.h>
#include <cache.h>
unsigned char sram_data[64] __attribute__((__aligned__(32)));

void exi_sram_read(void* data)
{
	unsigned long val;
	exi_select(0, 1, 3);
	val = 0x20000100;
	exi_write(0, &val, 4);
	exi_read(0, data, 64);
	exi_deselect(0);
}

void exi_sram_write(void* data)
{
	unsigned long val;
	exi_select(0, 1, 3);
	val = 0xA0000100;
	exi_write(0, &val, 4);
	exi_write(0, data, 64);
	exi_deselect(0);
}

void exi_ipl_write(unsigned long addr, unsigned long val)
{
	exi_select(0, 1, 3);
	exi_write(0, &addr, 4);
	exi_write(0, &val, 4);
	exi_deselect(0);
}

unsigned char rom_read(int addr)
{
	addr <<= 6;
	unsigned long val;
	exi_select(0, 1, 3);
	exi_write(0, &addr, 4);
	exi_read(0, &val, 4);
	exi_deselect(0);
	return val >> 24;
}

void rom_readn(int addr, unsigned char* d, int len)
{
	while (len)
	{
		int max = 1024 - (addr & 1023);
		if (len < max)
			max = len;

		addr <<= 6;
		exi_select(0, 1, 3);
		exi_write(0, &addr, 4);
		exi_read(0, d, max);
		exi_deselect(0);
		addr += max;
		len -= max;
	}
}

void rom_write(int addr, unsigned char c)
{
	unsigned long val;
	addr <<= 6;
	addr |= 0x80000000;
	val = c << 24;
	exi_select(0, 1, 3);
	exi_write(0, &addr, 4);
	exi_write(0, &val, 4);
	exi_deselect(0);
}

void s_putc(unsigned char c)
{
	unsigned long val, addr;
	addr = 0xA8000000;
	val = c << 24;
	exi_select(0, 1, 3);
	exi_write(0, &addr, 4);
	exi_write(0, &val, 4);
	exi_deselect(0);

	//	udelay(1000000/115200*20);
	mdelay(1);
}

void ipl_read(void* dst, unsigned long addr, int len)
{
	addr <<= 6;
	exi_select(0, 1, 3);
	exi_write(0, &addr, 4);
	exi_read(0, dst, len);
	exi_deselect(0);
}




void ipl_set_config(unsigned char c)
{
	//tmbinc/viper/ninjamod/ripper = 0xb0000000 | qoobpro/sx = 0xc0000000
	unsigned long val, addr;
	addr = 0xc0000000; 
	val = c << 24;
	exi_select(0, 1, 3);
	exi_write(0, &addr, 4);
	exi_write(0, &val, 4);
	exi_deselect(0);
}


void ipl_boot(int what)
{
	if (what < 0)
		ipl_set_config(0x2); // boot to original IPL
	else
		ipl_set_config(what << 4); // boot with prefix
	*(unsigned long*)0xcc003024 = 0;
	while (1);
}
