/*
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */
#include <linux/string.h>
#include <console.h>
#include <processor.h>
#include <exception.h>
#include <types.h>
#include <dvd.h>
#include "memlayout.h"
#include <font.h>
#include <vsprintf.h>
#include <cache.h>
/*
from tmbinc's original ipl, just modified slightly
*/
void load_dol(void* dol, int size)
{
	void (*entrypoint)();
	if ((*(unsigned long*)dol) > 0x1000)
	{
		write_font(150, 300, "loading binary file...");
		memcpy((void*)0x80003100, dol, size);
		entrypoint = (void(*)())0x80003100;
	}
	else
	{
		struct dol_s
		{
			unsigned long sec_pos[18];
			unsigned long sec_address[18];
			unsigned long sec_size[18];
			unsigned long bss_address, bss_size, entry_point;
		} *d = (struct dol_s *)dol;
		int i;

		for (i = 0; i < 18; ++i)
		{
			if (!d->sec_size[i])
				continue;
			memcpy((void*)d->sec_address[i], ((unsigned char*)dol) + d->sec_pos[i], d->sec_size[i]);
		}
		memset((void*)d->bss_address, 0, d->bss_size);
		entrypoint = (void(*)())d->entry_point;
	}
	mtmsr((mfmsr() | 2) & ~0x8000);
	flush_code((void*)0x80000000, 0x1800000);
	exception_close();
	entrypoint();
	write_font(100, 172, "returned from entrypoint - should never happen. but happens.");
	while (1);
}


/*
cheqmate
*/
void fn_load_dol_fn_inmem(void* dol, int size)
{
	void (*entrypoint)();

	struct dol_s
	{
		unsigned long sec_pos[18];
		unsigned long sec_address[18];
		unsigned long sec_size[18];
		unsigned long bss_address, bss_size, entry_point;
	} *d = (struct dol_s *)dol;

	int i;

	for (i = 0; i < 18; ++i)
	{
		if (!d->sec_size[i])
			continue;

		// copy section
		int nCount = d->sec_size[i];
		char* pDest = (char*)(void*)d->sec_address[i],* pSrc = (char*)((unsigned char*)dol) + d->sec_pos[i];
		while (nCount--)
			*pDest++ = *pSrc++;
	}

	// clear BSS
	int nCount = d->bss_size;
	char* pDest = (void*)d->bss_address;
	while (nCount--)
		*pDest++ = 0;//
	entrypoint = (void(*)())d->entry_point;
	mtmsr((mfmsr() | 2) & ~0x8000);
	entrypoint();
}


/*!	\fn			void load_dol_fn_inmem(void *dol, int size)
 *	
 *				simplyfied version of above function, doesnt use subroutines
 *				so it can be copied to another mem location. needed when
 *				uploading a dol that overlaps gcos itself
 *
 *	\param		none
 *	
 *	\return		none
 *	
 *	\note	
 *  cheqmate 2005	
 */
void load_dol_fn_inmem(void* dol, int size)
{
	u32 dwCodeSrc = (u32)fn_load_dol_fn_inmem;

//	printf("dolload code address: %p \n", &dwCodeSrc);

	exception_close();

	// copy dol loader code to safe buffer area
	memcpy((void*)0x81600000, (void*)dwCodeSrc, 0x200);

	flush_code(DOLUP_CODEBUFFER, 0x1000);
	//	flush_code(0x80000000, 0x1800000);

	void (*pLoadFn)(void*, int) = DOLUP_CODEBUFFER;

	pLoadFn(dol, size);
}
