/*
 * DOL loader over tcp/ip
 * Receives a DOL on port 4000 and executes it
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include "lwip/debug.h"
#include "lwip/stats.h"
#include "lwip/tcp.h"
#include <console.h>
#include <types.h>
#include <processor.h>
#include <elf.h>
#include <dol.h>


#include <memlayout.h>


#define TYPE_UNKNOWN 0
#define TYPE_DOL 1
#define TYPE_ELF 2

struct dolup_state
{
	struct dol_s
	{
		u32 sec_pos[18];
		u32 sec_address[18];
		u32 sec_size[18];
		u32 bss_address, bss_size, entry_point;
		u32 dummy[7];
	} dolhdr;
	u32 filepos;
	u32 filesize;
	int type;
	int retries;
};
/*-----------------------------------------------------------------------------------*/
static void conn_err(void* arg, err_t err)
{
	struct dolup_state* hs;

	hs = arg;
	mem_free(hs);
}
/*-----------------------------------------------------------------------------------*/

/*
static void
close_conn(struct tcp_pcb *pcb, struct dolup_state *hs)
{
	void (*entrypoint)();
	
	printf("DOLUP: connection CLOSE\n");
	tcp_arg(pcb, NULL);
	tcp_sent(pcb, NULL);
	tcp_recv(pcb, NULL);
	tcp_close(pcb);
	
	entrypoint = NULL;
	switch (hs->type) {
		case TYPE_DOL:
			entrypoint = (void(*)())hs->dolhdr.entry_point;
			break;
		case TYPE_ELF:
			entrypoint = (void(*)(void))load_elf_image((unsigned long)ELF_FILE_BUFFER);
			break;
		default:
			printf("unknown file type\n");
	}
	mem_free(hs);
	
	if (entrypoint != NULL) {
		// clear interrupt mask
		*(volatile unsigned long*)0xCC003004 = 0;

		exception_close();
		printf("starting execution at %p\n", entrypoint);
		entrypoint();
	}
}
*/


/*-----------------------------------------------------------------------------------*/
static void close_conn(struct tcp_pcb* pcb, struct dolup_state* hs)
{
	u32 dwSize = 0;
	u32 dwImageBaseMem = 0;

	void (*entrypoint)() = NULL;

	//printf("DOLUP: connection CLOSE\n");

	tcp_arg(pcb, NULL);
	tcp_sent(pcb, NULL);
	tcp_recv(pcb, NULL);
	tcp_close(pcb);

	// get image parameters
	switch (hs->type)
	{
	case TYPE_DOL:
		dwSize = hs->filesize;
		dwImageBaseMem = ELF_FILE_BUFFER;
		entrypoint = (void(*)())hs->dolhdr.entry_point;
		break;
	case TYPE_ELF:
		dwSize = hs->filesize;
		dwImageBaseMem = ELF_FILE_BUFFER;
		entrypoint = (void(*)(void))load_elf_image((unsigned long)ELF_FILE_BUFFER);
		break;
	default:
		break;
	}

	// show info
//	printf("downloaded image to %p size %p", &dwImageBaseMem, &dwSize);
	//printf("\nstarting at %p\n", entrypoint);

	mem_free(hs);

	if (entrypoint != NULL)
	{
		// clear interrupt mask
		*(volatile unsigned long*)0xCC003004 = 0;

#ifdef DOLUP_WAITFORBUTTON
		struct PAD pad;
		while (1)
		{
			PAD_ReadState(&pad, 0);
			if (pad.Digital.A)
			{
				// A continue
				break;
			}
			if (pad.Digital.B)
			{
				// B abort
				return;
			}
		}
#endif
		//-------------------------------------------------
		// run image
		//-------------------------------------------------
		switch (hs->type)
		{
		case TYPE_DOL:
			load_dol_fn_inmem((void*)dwImageBaseMem, dwSize);
			// load_dol((void *) dwImageBaseMem, dwSize);
			break;
		case TYPE_ELF:
			entrypoint();
			break;
		default:
			break;
		}
	}
}

/*-----------------------------------------------------------------------------------*/
static err_t dolup_poll(void* arg, struct tcp_pcb* pcb)
{
	struct dolup_state* hs;

	hs = arg;

	// printf("Polll\n");
	if (hs == NULL)
	{
//		printf("Null, close\n");
		tcp_abort(pcb);
		return ERR_ABRT;
	}
	else
	{
		++hs->retries;
		if (hs->retries == 40)
		{
			tcp_abort(pcb);
			return ERR_ABRT;
		}
		//		send_data(pcb, hs);
	}

	return ERR_OK;
}
/*-----------------------------------------------------------------------------------*/
#if 0
static err_t
dolup_sent(void *arg, struct tcp_pcb *pcb, u16_t len)
{
	return ERR_OK;
}
#endif
/*-----------------------------------------------------------------------------------*/
static err_t dolup_recv(void* arg, struct tcp_pcb* pcb, struct pbuf* p, err_t err)
{
	struct dolup_state* hs;

	hs = arg;

	if (err == ERR_OK && p != NULL)
	{
		// Inform TCP that we have taken the data.
		tcp_recved(pcb, p->tot_len);

		u8* data = p->payload;
		int len = p->tot_len;
		while (len > 0)
		{
			//-------------------------------------
			// get header data (0h-100h)
			//-------------------------------------
			if (hs->filepos < 0x100)
			{
//				printf("H");

				u8* header = (u8*)(&hs->dolhdr);
				int tmp_len = len;

				if (hs->filepos + tmp_len > 0x100)
				{
					tmp_len = 0x100 - hs->filepos;
				}

				memcpy(header + hs->filepos, data, tmp_len);
				hs->filepos += tmp_len;

				// end of header
				if (hs->filepos == 0x100)
				{
					//-----------------------------------------
					// elf or dol ?
					//-----------------------------------------
					if (valid_elf_image((u32)header))
					{
						// prepare elf image download
						hs->type = TYPE_ELF;
						hs->filesize = 0;	// no decoding
						memcpy(ELF_FILE_BUFFER, header, 0x100);
					}
					else
					{
						int i;
						// prepare dol image download
						hs->type = TYPE_DOL;
						hs->filesize = 0x100;

						// copy header
						memcpy(ELF_FILE_BUFFER, header, 0x100);

						// calculate dol size from section header
						for (i = 0; i < 18; i++)
						{
							u32 sec_end = hs->dolhdr.sec_pos[i] + hs->dolhdr.sec_size[i];
							if (sec_end > hs->filesize)
							{
								hs->filesize = sec_end;
							}
						}
					}
				}
				data += tmp_len;
				len -= tmp_len;
			} 
			//-------------------------------------
			// get payload data ( > 100h)
			//-------------------------------------
			else
			{
				switch (hs->type)
				{
				case TYPE_DOL:
					{
//						printf("D");

						// copy data into temp buffer
						memcpy(ELF_FILE_BUFFER + hs->filepos, data, len);
						hs->filepos += len;
						data += len;
						len = 0;
						break;

						/*
																						// determine VMA of current data chunk by checking section headers
																						int i, found=0;
																						for (i=0; i<18 && !found ; i++) {
																							if ((hs->dolhdr.sec_pos[i] <= hs->filepos) && (hs->dolhdr.sec_pos[i]+hs->dolhdr.sec_size[i] > hs->filepos)) {
																								u32 tmp_offset = hs->filepos - hs->dolhdr.sec_pos[i];
																								u32 tmp_len = len;
																								if ( tmp_offset + tmp_len > hs->dolhdr.sec_size[i] ) {
																									tmp_len = hs->dolhdr.sec_size[i] - tmp_offset;
																								}
																								printf("X");
																								memcpy((void*)(hs->dolhdr.sec_address[i] + tmp_offset), data, tmp_len);
																								flush_code((void*)(hs->dolhdr.sec_address[i] + tmp_offset), tmp_len);
																								data+=tmp_len;
																								len-=tmp_len;
																								hs->filepos+=tmp_len;
																								found=1;
																							}
																						}
																						if (!found) {
																							// if the filepos was not found in the dol header, advance the file pointer by one byte. there could be some
																							// alignment between sections, for example for DVD reads
																							printf("Z");
																							data++;
																							len--;
																							hs->filepos++;
																						}
																						*/
						break;
					}
				default:
					{
						// copy others into buffer
						memcpy(ELF_FILE_BUFFER + hs->filepos, data, len);
						hs->filepos += len;
						data += len;
						len = 0;
//						printf("U");
						break;
					}
				}
			}
		}
	}

	if (p != NULL)
	{
		pbuf_free(p);
	}

	if (err == ERR_OK)
	{
		if ((hs->filepos >= hs->filesize && hs->filesize > 0) || (hs->type == TYPE_ELF && p == NULL))
		{
//			printf("\n");
			close_conn(pcb, hs);
		}
		else
		{
			// printf(".");
		}
	}

	return ERR_OK;
}


/*-----------------------------------------------------------------------------------*/
static err_t dolup_accept(void* arg, struct tcp_pcb* pcb, err_t err)
{
	struct dolup_state* hs;

	tcp_setprio(pcb, TCP_PRIO_MIN);

	/* Allocate memory for the structure that holds the state of the connection. */
	//hs = mem_malloc(sizeof(struct dolup_state));
	//by VTrucco. Working much better for me
	hs = mem_malloc(sizeof(struct dolup_state) * 100);

	if (hs == NULL)
	{
//		printf("dolup_accept: Out of memory\n");
		return ERR_MEM;
	}

//	printf("dolup: accept\n");
	hs->filepos = 0;
	hs->filesize = 0;
	hs->type = TYPE_UNKNOWN;

	/* Tell TCP that this is the structure we wish to be passed for our
		callbacks. */
	tcp_arg(pcb, hs);

	/* Tell TCP that we wish to be informed of incoming data by a call
		to the dolup_recv() function. */
	tcp_recv(pcb, dolup_recv);

	tcp_err(pcb, conn_err);

	tcp_poll(pcb, dolup_poll, 4);
	return ERR_OK;
}
/*-----------------------------------------------------------------------------------*/
void dolup_init(void)
{
	struct tcp_pcb* pcb;

	pcb = tcp_new();
	tcp_bind(pcb, IP_ADDR_ANY, 4000);
	pcb = tcp_listen(pcb);
	tcp_accept(pcb, dolup_accept);
}
