#include "dvd.h"
#include "menu/draw.h"
#include "linux/string.h"
#include "processor.h"
#include "exi.h"
#include "console.h"
#include "pad.h"
#include "video.h"
#include "color.h"
#include "time.h"
#include "sram.h"
#include <irq.h>
#include "elf.h"
#include <exception.h>
#include <cache.h>
#include "iso9660qoob.h"
#include "GC_dvd.h"
#include "menu/menu.h" 
#include "pad.h"
#include "font.h"
#include <dvd.h>
#include "console.h"
#include "main.h"
#include "menu/draw.h"
#include "lowmem.h"
#include <anaconda.h>
#include <setupdrive.h>
unsigned long* fba = (void*)MEM_FB;
extern long* fb;
extern unsigned char drivemodel;
extern unsigned char drivemodelq;
extern long blit_lookup[4];
extern long blit_lookup_inv[4];
extern int font_offset[256], font_size[256], font_height;
extern u32* frameBuffer;
extern unsigned char sector_buffer[2048] __attribute__((aligned(32)));

/* QOOB STUFF */
/* ------------------------------------------------------------------ */

#define log __ieee754_log
#define sqrtf __ieee754_sqrtf
#include <math.h>
int audio_len;
int calc_rq(void);
/* QOOB STUFF */
/* ------------------------------------------------------------------ */

int pressed = 0, selection = 0, redraw = 1;
int offset = 0;


void set_status(char *message)
{
	write_font(125, 300,message);
}
void fatal_read_error()
{
	set_status("FATAL READ ERROR!");
	while (1);
}


void clear_screen()
{
	unsigned long* video_fb = (void*)MEM_FB;
	int pixel = 320 * 576;
	while (pixel--)
		*video_fb++ = blit_lookup[0];
}

int is_unicode;


int files;

int last_current_dir = -1;

int read_direntry(unsigned char* direntry)
{
	int nrb = *direntry++;
	++direntry;

	int sector;

	direntry += 4;
	sector = (*direntry++) << 24;
	sector |= (*direntry++) << 16;
	sector |= (*direntry++) << 8;
	sector |= (*direntry++);	

	int size;

	direntry += 4;

	size = (*direntry++) << 24;
	size |= (*direntry++) << 16;
	size |= (*direntry++) << 8;
	size |= (*direntry++);

	direntry += 7; // skip date

	int flags = *direntry++;
	++direntry; ++direntry; direntry += 4;

	int nl = *direntry++;

	char* name = file[files].name;

	file[files].sector = sector;
	file[files].size = size;
	file[files].flags = flags;

	if ((nl == 1) && (direntry[0] == 1)) // ".."
	{
		file[files].name[0] = 0;
		if (last_current_dir != sector)
			files++;
	}
	else if ((nl == 1) && (direntry[0] == 0))
	{
		last_current_dir = sector;
	}
	else
	{
		if (is_unicode)
		{
			int i;
			for (i = 0; i < (nl / 2); ++i)
				name[i] = direntry[i * 2 + 1];
			name[i] = 0;
			nl = i;
		}
		else
		{
			memcpy(name, direntry, nl);
			name[nl] = 0;
		}

		if (!(flags & 2))
		{
			if (name[nl - 2] == ';')
				name[nl - 2] = 0;

			int i = nl;
			while (i >= 0)
				if (name[i] == '.')
					break;
				else
					--i;

			++i;

		}
		else
		{
			name[nl++] = '/';
			name[nl] = 0;
		}

		files++;
	}

	return nrb;
}



void read_directory(int sector, int len)
{
	read_sector(sector_buffer, sector);

	int ptr = 0;
	files = 0;

	while (len > 0)
	{
		ptr += read_direntry(sector_buffer + ptr);
		if (!sector_buffer[ptr])
		{
			len -= 2048;
			read_sector(sector_buffer, ++sector);
			ptr = 0;
		}
	}
}



//toreplace the iso9660 showdir thing
void show_files(int offset, int selection)
{
	int x = 75;
	int y = 130;

	int i;
	for (i = offset; (i < (offset + PAGESIZE)) && (i < files); ++i, y += font_height)
	{
		//writex(x, y, 420, font_height, file[i].name[0] ? file[i].name : "..", (selection == i) ? blit_lookup : blit_faded);
		if(selection == i)
			drawHighlight(frameBuffer, x-35, y+3,270);
		write_font(x,y,file[i].name[0] ? file[i].name : "..");
	}
}

int doiso9660()
{
	set_status("Reading DVD...");

	int sector = 16;
	static unsigned char buffer[2048] __attribute__((aligned(32)));

	struct pvd_s* pvd = 0;
	struct pvd_s* svd = 0;
	//	printf("Reading in DVD...\n");
	while (sector < 32)
	{
		if (read_sector(buffer, sector))
			fatal_read_error();
		if (!memcmp(((struct pvd_s *)buffer)->id, "\2CD001\1", 8))
		{
			svd = (void*)buffer;
			break;
		}
		++sector;
	}

	if (!svd)
	{
		sector = 16;
		while (sector < 32)
		{
			if (read_sector(buffer, sector))
				fatal_read_error();

			if (!memcmp(((struct pvd_s *)buffer)->id, "\1CD001\1", 8))
			{
				pvd = (void*)buffer;
				break;
			}
			++sector;
		}
	}

	if ((!pvd) && (!svd))
	{
		set_status("No ISO9660 DVD Found!");
		menu_redo();
	}

	files = 0;
	if (svd)
	{
		is_unicode = 1;	
		read_direntry(svd->root_direntry);
	}
	else
	{
		is_unicode = 0;
		read_direntry(pvd->root_direntry);
	}

	// enter root
	read_directory(file[0].sector, file[0].size);

	pressed = 0;
	selection = 0;
	redraw = 1;
	offset = 0;

	iso9660qoob_redraw();

	return 0;
}

void iso9660qoob_redraw()
{
	video_wait_vsync();
	clearCentre();
	show_files(offset, selection);
}

void iso9660qoob_down()
{
	GC_Sleep(35);
	selection++;

	if (selection == files)
	{
		selection = 0;
		offset = 0;
	}

	if ((selection - offset) >= PAGESIZE)
		offset += PAGESIZE;

	iso9660qoob_redraw();
}

void iso9660qoob_up()
{
	GC_Sleep(35);
	selection--;
	if (selection < 0)
	{
		selection = files - 1;
		offset = selection - PAGESIZE + 1;
	}

	if (selection < offset)
		offset -= PAGESIZE;

	if (offset < 0)
		offset = 0;

	iso9660qoob_redraw();
}


void iso9660qoob_downX()
{
	if (selection == files)
	{
		selection = 0;
		offset = 0;
	}
	if (selection <= (files - 7))
	{
 		selection+=6;
	}
	
	if ((selection - offset) >= PAGESIZE)
		offset += PAGESIZE;

	iso9660qoob_redraw();
}

void iso9660qoob_upY()
{
	selection-=6;


	if (selection < 0)
	{
		selection = files - 1;
		offset = selection - PAGESIZE + 1;
	}

	if (selection < offset)
		offset -= PAGESIZE;

	if (offset < 0)
		offset = 0;

	iso9660qoob_redraw();
}

int iso9660qoob_ok()
{
# if defined(PSO) && defined(MOD)
#  error "cannot define both PSO and MOD"
# endif

# if defined(MOD)
#define addr		0x80C00000; //81300000 build
# endif
# if defined(PSO)
#define addr		0x81000000 //80d00000 build
# endif

	//calc_rq();
	if (file[selection].flags & 2) // dir
	{
		read_directory(file[selection].sector, file[selection].size);
		selection = offset = 0;
		iso9660qoob_redraw();
		return 0;
	} 

	u8* p;
	u8* name = file[selection].name;
	p = strrchr(name, '.');
	if (p && strnicmp(p + 1, "DOL", 3) == 0)
	{
		//For a DOL
		void* dolfile = (void*)addr;
		// Now lets read in the dol to the right spot.
		read_safe(dolfile, file[selection].size, (file[selection].sector * 2048));
		clearCentre();
		load_dol(dolfile, file[selection].size);
		return 0;
	}

	if (p && strnicmp(p + 1, "ELF", 3) == 0)
	{
		//For a ELF
		void (*entrypoint)() = NULL;
		void* elffile = (void*)addr;
		// Now lets read in the elf to the right spot.
		read_safe(elffile, file[selection].size, (file[selection].sector * 2048));
		clearCentre();
		entrypoint = (void(*)(void))load_elf_image(&elffile);
		entrypoint();
		return 0;
	}

	if ((p && strnicmp(p + 1, "GCM", 3) == 0) || (p && strnicmp(p + 1, "ISO", 3) == 0))
	{
	     long theOffset = (file[selection].sector * 2048);
		 clearCentre();
	     if((theOffset % 0x20000)!= 0 )
	     {
	     	write_font(120,150,"Not on 128KByte Boundary!");
	     	GC_Sleep(5000);
	     	return;
	     }
	     else
	     {
	     	dvd_set_offset(theOffset);
	     	FinalizeBoot();
     	 }
	     return 0;
	}
	else
	{
		clearCentre();
		write_font(150,150,"File Type Unknown");
		GC_Sleep(1000);
		return 0;
	}
	if (selection < (files - 1))
	{
	}
}
void isoHandler()
{
            int waitrelease = 0;
            clearCentre();
			doiso9660();

			PAD pad;

			while (1)
			{
				PAD_ReadState(&pad, 0);

				int isup = 0, isdown = 0;

				if (pad.Digital.Up || (pad.Analog.Y < -50))
					isup = 1;
				if (pad.Digital.Down || (pad.Analog.Y > 50))
					isdown = 1;

				if (pad.Digital.A && (!waitrelease))
				{
					//The user must have release the button 
					do
					{
						PAD_ReadState(&pad, 0);
					}
					while (pad.Digital.A);
					clearCentre();
					iso9660qoob_ok();
					waitrelease = 1;
					iso9660qoob_up();
				}
				else if (pad.Digital.B)
				{
					//The user must have release the button 
					do
					{
						PAD_ReadState(&pad, 0);
					}
					while (pad.Digital.B);
					menu_redo();
					break;
				}
				if (pad.Digital.X && (!waitrelease))
				{
					//The user must have release the button 
					do
					{
						PAD_ReadState(&pad, 0);
					}
					while (pad.Digital.X);

					iso9660qoob_downX();

					waitrelease = 1;
				} 
				if (pad.Digital.Y && (!waitrelease))
				{
					//The user must have release the button 
					do
					{
						PAD_ReadState(&pad, 0);
					}
					while (pad.Digital.Y);

					iso9660qoob_upY();

					waitrelease = 1;
				} 
				if (isup)
				{
					iso9660qoob_up();
					//		waitrelease = 1;
				}
				if (isdown)
				{
					iso9660qoob_down();

					//		 waitrelease = 1;
				}
				if (!(isup || isdown || pad.Digital.A || pad.Digital.B))
					waitrelease = 0;
			}
}
