/* exception.c - exception handling

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
 if not, write to the 
 Free Software Foundation, Inc., 
 59 Temple Place, 
 Suite 330, 
 Boston, 
 MA 02111-1307 USA
*/

#include <exception.h>
#include <console.h>
#include <linux/string.h>
#include <irq.h>
#include <cache.h>
#include <font.h>
#include <color.h>
#include "menu/draw.h"
extern u32* frameBuffer;
extern char exception_start, exception_end, exception_patch;
extern void exception_handler_default();
extern void exception_handler_decrementer();
extern void exception_handler_system_call();
extern void exception_handler_external();

unsigned long exception_location[15] =
{
	0x00000100, 0x00000200, 0x00000300, 0x00000400, 0x00000500, 0x00000600, 0x00000700, 0x00000800, 0x00000900, 0x00000C00, 0x00000D00, 0x00000F00, 0x00001300, 0x00001400, 0x00001700
};

const char* exception_name[15] =
{
	"System Reset", "Machine Check", "DSI", "ISI", "Interrupt", "Alignment", "Program", "Floating Point", "Decrementer", "System Call", "Trace", "Performance", "IABR", "Reserved", "Thermal"
};

void exception_init()
{
	int i;
	for (i = 0; i < 15; ++i)
	{
		void* address = (void*)(exception_location[i] | 0x80000000);
		memcpy(address, &exception_start, &exception_end - &exception_start);
		// fix the "li" to load the correct exception number
		*(unsigned long*)(&exception_patch - &exception_start + (int)address) |= i;
		flush_code(address, &exception_end - &exception_start);
		exception_set_handler(i, exception_handler_default);

		//		*(long*) address = 0x4c000064;
		flush_code(address, 0x100);
	}

	exception_set_handler(8, exception_handler_decrementer);
	exception_set_handler(9, exception_handler_system_call);
	exception_set_handler(4, irq_ext);

	CONTEXT_CURRENT = 0x80002000;
	CONTEXT_CURRENT_PHYS = 0x2000;
}

void exception_close()
{
	int i;
	for (i = 0; i < 15; ++i)
	{
		void* address = (void*)(exception_location[i] | 0x80000000);
		*(long*)address = 0x4c000064;
		flush_code(address, 0x100);
	}
	mtmsr(mfmsr() & ~0x8000); // EE
	mtmsr(mfmsr() | 0x2002);  // FP, RI
}

void exception_set_handler(int exception, void (*handler)(int, struct context_s*))
{
	((void**)0x80003000)[exception] = handler;
}

void exception_handler_default(int exception)
{
	struct context_s* c = (struct context_s*)CONTEXT_CURRENT;
	//	clearScreen(frameBuffer, COLOR_BLACK);
	write_font(500, 500, exception_name[exception]);
/*
	printf("GPR00 %08lx GPR08 %08lx GPR16 %08lx GPR24 %08lx\n", c->GPR[0], c->GPR[8], c->GPR[16], c->GPR[24]);
	printf("GPR01 %08lx GPR09 %08lx GPR17 %08lx GPR25 %08lx\n", c->GPR[1], c->GPR[9], c->GPR[17], c->GPR[25]);
	printf("GPR02 %08lx GPR10 %08lx GPR18 %08lx GPR26 %08lx\n", c->GPR[2], c->GPR[10], c->GPR[18], c->GPR[26]);
	printf("GPR03 %08lx GPR11 %08lx GPR19 %08lx GPR27 %08lx\n", c->GPR[3], c->GPR[11], c->GPR[19], c->GPR[27]);
	printf("GPR04 %08lx GPR12 %08lx GPR20 %08lx GPR28 %08lx\n", c->GPR[4], c->GPR[12], c->GPR[20], c->GPR[28]);
	printf("GPR05 %08lx GPR13 %08lx GPR21 %08lx GPR29 %08lx\n", c->GPR[5], c->GPR[13], c->GPR[21], c->GPR[29]);
	printf("GPR06 %08lx GPR14 %08lx GPR22 %08lx GPR30 %08lx\n", c->GPR[6], c->GPR[14], c->GPR[22], c->GPR[30]);
	printf("GPR07 %08lx GPR15 %08lx GPR23 %08lx GPR31 %08lx\n", c->GPR[7], c->GPR[15], c->GPR[23], c->GPR[31]);
	printf("LR %08lx SRR0 %08lx %08lx\n", c->LR, c->SRR0, c->SRR1);
	printf("DAR: %08lx DSISR %08lx\n", mfspr(19), mfspr(18));
*/
	if ((exception == 2) || (exception == 7) || (exception == 6)) // DSI
	{
		int i;
		unsigned long* address = (unsigned long*)c->SRR0;
	/*	printf("CODE DUMP:\n");
		for (i = 0; i < 12; i += 4)
			printf("%08lx %08lx %08lx %08lx\n", address[i], address[i + 1], address[i + 2], address[i + 3]);
	*/}
	while (1);
}
