/* time.c - clock functions for the ipl

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
 if not, write to the 
 Free Software Foundation, Inc., 
 59 Temple Place, 
 Suite 330, 
 Boston, 
 MA 02111-1307 USA
*/
#include "processor.h"

#define TB_CLOCK  40500000

unsigned long tb_diff_sec(tb_t* end, tb_t* start)
{
	unsigned long upper, lower;
	upper = end->u - start->u;
	if (start->l > end->l)
		upper--;
	lower = end->l - start->l;
	return ((upper * ((unsigned long)0x80000000 / (TB_CLOCK / 2))) + (lower / TB_CLOCK));
}

unsigned long tb_diff_msec(tb_t* end, tb_t* start)
{
	unsigned long upper, lower;
	upper = end->u - start->u;
	if (start->l > end->l)
		upper--;
	lower = end->l - start->l;
	return ((upper * ((unsigned long)0x80000000 / (TB_CLOCK / 2000))) + (lower / (TB_CLOCK / 1000)));
}

unsigned long tb_diff_usec(tb_t* end, tb_t* start)
{
	unsigned long upper, lower;
	upper = end->u - start->u;
	if (start->l > end->l)
		upper--;
	lower = end->l - start->l;
	return ((upper * ((unsigned long)0x80000000 / (TB_CLOCK / 2000000))) + (lower / (TB_CLOCK / 1000000)));
}

unsigned long tb_diff(tb_t* end, tb_t* start)
{
	unsigned long lower;
	lower = end->l - start->l;
	return lower;
}

void udelay(unsigned int us)
{
	tb_t start, end;
	mftb(&start);
	while (1)
	{
		mftb(&end);
		if (tb_diff_usec(&end, &start) >= us)
			break;
	}
}

void mdelay(unsigned int ms)
{
	tb_t start, end;
	mftb(&start);
	while (1)
	{
		mftb(&end);
		if (tb_diff_msec(&end, &start) >= ms)
			break;
	}
}

void delay(unsigned int s)
{
	tb_t start, end;
	mftb(&start);
	while (1)
	{
		mftb(&end);
		if (tb_diff_sec(&end, &start) >= s)
			break;
	}
}

