/*
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */
#include "draw.h"
#include "types.h"
#include "color.h"
#include "GC_Dvd.h"
#include "video.h"
#include "main.h"
#include <rgb.h>
#include <menu/selection_middle.h>
#include <menu/selection_end_right.h>
#include <menu/selection_end_left.h>

#include "math.h"	
u32* frameBuffer;
#include <menu/gradient.h>

void menuDrawHLine(u32* curFrameBuffer, int x1, int x2, int y, u32 color)
{
	int i;
	y = SCR_WIDTH * y / 2;
	x1 = x1 / 2;
	x2 = x2 / 2;
	for (i = x1; i <= x2; i++)
		curFrameBuffer[y + i] = color;
}

void menuDrawVLine(u32* curFrameBuffer, int x, int y1, int y2, u32 color)
{
	int i;
	x = x / 2;
	for (i = y1; i <= y2; i++)
		curFrameBuffer[x + SCR_WIDTH * i / 2] = color;
}

void menuDrawBox(u32* curFrameBuffer, int x1, int y1, int x2, int y2, u32 color)
{
	menuDrawHLine(curFrameBuffer, x1, x2, y1, color);
	menuDrawHLine(curFrameBuffer, x1, x2, y2, color);
	menuDrawVLine(curFrameBuffer, x1, y1, y2, color);
	menuDrawVLine(curFrameBuffer, x2, y1, y2, color);
}

void clearScreen(u32* curFrameBuffer, u32 Color)
{
	int i, j, x, y;

	x = SCR_WIDTH / 2;

	for (j = 0; j <= SCR_HEIGHT; j++)
	{
		y = SCR_WIDTH * j / 2;
		for (i = 0; i <= x; i++)
			curFrameBuffer[y + i] = Color;
	}
}
void drawFilledBox(u32* dst, s32 x1, s32 y1, s32 x2, s32 y2, u32 color)
{
	s32 i,cpH = y2 - y1;
	for (i = 0; i < cpH; i++)
		menuDrawHLine(dst, x1, x2, y1 + i, color);
}
 
void menuDrawBanner(u32 *curFrameBuffer,u32 *banner ,int scrX, int scrY, int iZoom)
{
	int w = 96;
	int h = 32;
	int bufX = 0;
	int bufY = 0;
    int x, y, i, j, zx, zy, offset;

    scrX /= 2;
    w /= 2;
    for (y = scrY; y < scrY + h; y++) {
	for (x = scrX; x < scrX + w; x++) {
	    offset = x - scrX + bufX + ((y - scrY + bufY) * 96) /2;
		zx = scrX + (x - scrX) * iZoom;
		zy = scrY + (y - scrY) * iZoom;
		for (i = 0; i < iZoom; i++) {
		    for (j = 0; j < iZoom; j++) {
		    curFrameBuffer[(zx + i) + SCR_WIDTH * (zy + j) /2] = banner[offset];
		    }
		}
	}
    }
}

void drawBitmap(u32* curFrameBuffer, const unsigned long* image, int scrX, int scrY, int w, int h)
{
	int i, j;
	int current = 0;

	for (j = 0 + scrY; j < h + scrY; j++)
	{
		for (i = 0; i < w/2; i++)
		{
			
			if (image[current] != COLOR_WHITE)
				curFrameBuffer[320 * j + i + scrX] = image[current];
			current++;
		}
	}
}

void drawBitmapNoDodge(u32* curFrameBuffer, const unsigned long* image, int scrX, int scrY, int w, int h)
{
	int i, j;
	int current = 0;

	for (j = 0 + scrY; j < h + scrY; j++)
	{
		for (i = 0; i < w/2; i++)
		{
			curFrameBuffer[320 * j + i + scrX] = image[current];
			current++;
		}
	}
}


void drawBitmapDodge(u32* curFrameBuffer, const unsigned long* image, int scrX, int scrY, int w, int h,u32 col)
{
	int i, j;
	int current = 0;

	for (j = 0 + scrY; j < h + scrY; j++)
	{
		for (i = 0; i < w / 2; i++)
		{
			if (image[current] != col)
				curFrameBuffer[320 * j + i + scrX] = image[current];
			current++;
		}
	}
}


void drawBitmapSpecial(u32* curFrameBuffer, const unsigned long* image, int scrX, int scrY, int w, int h)
{
	int i, j;
	int current = 0;

	for (j = 0 + scrY; j < h + scrY; j++)
	{
		for (i = 0; i < w/2; i++)
		{
			if (image[current] != 0x69D469EA)
			{
				if(image[current] == 0x80BF69BD)
				{	
					curFrameBuffer[320 * j + i + scrX] = COLOR_BLACK;
				}
				if(image[current] == 0x69BF80BD)
				{	
					curFrameBuffer[320 * j + i + scrX] = COLOR_BLACK;
				}
				
				if((image[current] != 0x80BF69BD) && (image[current] != 0x69BF80BD))
					curFrameBuffer[320 * j + i + scrX] = image[current];
			}
			current++;
		}
	}
}

void drawHighlight(u32* curFrameBuffer, int sx, int sy,int endX)
{
	drawBitmapSpecial(frameBuffer, selection_end_left_Bitmap, sx-2, sy, 6, 20);
	drawBitmapSpecial(frameBuffer, selection_end_right_Bitmap, endX, sy, 6, 20);
	int i = 0;  
	for(i = (sx); i<(endX); i++)
		drawBitmapDodge(frameBuffer, selection_middle_Bitmap, i, sy, 2, 20,COLOR_RED);

}

void drawGradient(u32* curFrameBuffer, int sx, int sy, int endX)
{
	int i = 0;
	for(i = sx; i<=endX; i++)
		drawBitmapDodge(frameBuffer, gradient_Bitmap,i,sy, 2, 310,COLOR_RED);
}

void clearCentre()
{
	drawGradient(frameBuffer, 32, 110,287);
	
}

void drawBitmapInv(u32* curFrameBuffer, const unsigned long* image, int scrX, int scrY, int w, int h)
{
	int i, j;
	int current = 0;

	for (j = h + scrY; j > 0; j--)
	{
		for (i = w / 2; i > 0; i--)
		{
			if (image[current] != COLOR_WHITE)
				curFrameBuffer[320 * j + i + scrX] = image[current];
			current++;
		}
	}
}
