/**
 ** MULTI-GAME DISC MAKER FOR GCOS Wii EDITION vDVD9
 ** ===============================================
 **/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h> 
#include <unistd.h>
#include <string.h>
#include <limits.h>
#include <io.h>
#include <fcntl.h>
#include "GCM.H"
#define HEADER_LEN		64
#define	MAX_GAMES		128
#define SWAP32(A) ((((A) & 0xff000000) >> 24) | (((A) & 0x00ff0000) >> 8) | (((A) & 0x0000ff00) << 8) | (((A) & 0x000000ff) << 24)) 

static unsigned char header[]={
	0x47,0x43,0x4F,0x50,0x44,0x56,0x44,0x39,0x01,0x00,0x00,0x00,0x00, //"GCOPDVD9" with streaming byte set to 01
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0xc2,0x33,0x9f,0x3d,0x47,0x43,0x4f,0x53,0x20,0x4d,0x75, //0xC2339f3d Magic // Title
	0x6c,0x74,0x69,0x47,0x61,0x6d,0x65,0x20,0x44,0x56,0x44,0x20,0x28, //Title
	0x43,0x29,0x20,0x47,0x43,0x4f,0x53,0x20,0x54,0x45,0x41,0x4d,0x4d  //Title
};

unsigned int gameOffsets[MAX_GAMES]; //endian swapped shite for win32
unsigned int gameOffsetsnorm[MAX_GAMES]; //big endian
unsigned int gameSizes[MAX_GAMES];
unsigned int paddingSizes[MAX_GAMES];
unsigned char readBuf[4194304*2];
int optimize(unsigned int size);

void banner() {
	fprintf(stdout, "\n  ============================================== \n"
					  "      GCOS SELFBOOT MULTI GAMEDISC MAKER v0.4F+  \n"
					  "            (Now with DVD9 & GC Support)         \n"
					  "  ============================================== \n\n");
}

void usage(char *prog) {
	fprintf(stderr, "  Usage  : %s <output> <region> <game1> <game2> <etc>\n"
					"  Example: %s multidisc.gcm PAL zelda.gcm mario.gcm\n"
					"  Example: %s multidisc.gcm JAP zelda.gcm mario.gcm\n"
					"  Example: %s multidisc.gcm USA zelda.gcm mario.gcm\n\n", prog,prog,prog,prog);
}

int main(int argc, char *argv[]) {
	FILE *fileOut, *fileIn;
	struct stat fileInfo;
	long long curOffset = 0x40000;

	unsigned int numGames = 0;
	int i = 0, j = 0, ok = 0;
	unsigned int nBytes = 0;
	unsigned int fsize = 0x40000;
	unsigned int finalOffset = 0;

	banner();
	
	if (argc < 3) {
		usage(argv[0]);
		return -1;
	}

	memset(gameOffsets, 0x00, (MAX_GAMES * 4));
	
	for (i = 3; i < argc; i++) {
		if (stat(argv[i], &fileInfo) == -1) {
			printf("ERROR: File not found! (%s)\n", argv[i]);
			return -1;
		}
		
		gameOffsetsnorm[i - 3] = curOffset>>2;
		gameOffsets[i - 3] = SWAP32(gameOffsetsnorm[i - 3]);
		numGames++;
		
		curOffset += (fileInfo.st_size);
		gameSizes[i-3] = fileInfo.st_size;
		paddingSizes[i-3] = (0x40000 - ((curOffset) % 0x20000));
        curOffset += paddingSizes[i-3];
	}

	if ((fileOut = fopen(argv[1], "wb")) == NULL) {
		fprintf(stderr, "    - ERROR opening output file! (%s)\n\n", argv[1]);
		return -1;
	}

	// Write Header
	fwrite(header, 64, 1, fileOut);
	
	// Write Offsets Table
	for (i = 0; i < numGames; i++) {
		fwrite(&gameOffsets[i], 4, 1, fileOut);	
	}
	
	// Modify Data for Region prior to writing
	if ( (strncmp((const char*)argv[2], "PAL", 3) == 0) ||
		(strncmp((const char*)argv[2], "pal", 3) == 0) )
	{
		printf("  Setting ISO to PAL\n");
		GCMdata[0x45b] = 0x02;
		ok = 1;
	}
	if ( (strncmp((const char*)argv[2], "USA", 3) == 0) ||
		(strncmp((const char*)argv[2], "usa", 3) == 0) )
	{
		printf("  Setting ISO to USA\n");
		GCMdata[0x45b] = 0x01;
		ok = 1;
	}
	if ( (strncmp((const char*)argv[2], "JAP", 3) == 0) ||
		(strncmp((const char*)argv[2], "jap", 3) == 0) )
	{
		printf("  Setting ISO to JAP\n");
		GCMdata[0x45b] = 0x00;
		ok = 1;
	}
	if(!ok)
	{
		printf("  Specify Region!\n");
		usage(argv[0]);
		return -1;
	}
	
	// Write GCOS in there for self-boot
	fwrite(GCMdata+64+(numGames*4), GCMDATA_LEN, 1, fileOut);
	
	// Write dummy data up to just before first GCM
	for (i = 0; i < (0x40000 - ((numGames * 4) + 64 + GCMDATA_LEN)); i++) {
		fputc(0x00, fileOut);
	}
	
	long long total_written = ftell(fileOut);
	long long padding = 0;
	int readsize = 0;
	long long SizeToRead = 0;
	// Add Games
	for (i = 0; i < numGames; i++) {
		printf("\n [+] Adding: \"%s\" .. ", argv[3 + i]);	fflush(stdout);
		fileIn = fopen64(argv[3 + i], "rb");        // open the current file we are going to add
        
        // calculate a good even number of bytes to read and write with
        //readsize = optimize(gameSizes[i]);
        readsize = 4194304*2;
     //   printf("Using %d chunks\n",readsize);
        SizeToRead = gameSizes[i];
        while(SizeToRead>=readsize) {
            memset(readBuf, 0x00,  readsize);		
			fread(readBuf, readsize, 1, fileIn);
			fwrite(readBuf,  readsize, 1, fileOut);
			SizeToRead-=readsize;
        }
        if(SizeToRead>0) {   //write out the end
            memset(readBuf, 0x00,  SizeToRead);		
			fread(readBuf, SizeToRead, 1, fileIn);
			fwrite(readBuf,  SizeToRead, 1, fileOut);
        }
		fclose(fileIn);

        if(i != numGames-1)
        {
        //      printf("\nPadding with %d bytes\n",paddingSizes[i]);
		    // Add proper amount of padding ..
		    for (j = 0; j < paddingSizes[i]; j++) {
			    fputc(0xFF, fileOut);
		    }
        }
        printf("DONE! (Wrote %d bytes)\n", gameSizes[i]);
        
	}

	fclose(fileOut);

	printf("\n\n MultiGame Disc Successfully Created!\n");
	
	return 0;
	
}


int optimize(unsigned int size) {
    int i = 0;
    for(i = 4194304; i>=1024; i/=2)
    {
          if((size % i) == 0)
          return i;
    }
    return 4096;
}
